"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlueSchemaRegistry = exports.ERROR = exports.SchemaCompatibilityType = exports.SchemaType = void 0;
const node_crypto_1 = __importDefault(require("node:crypto"));
const node_zlib_1 = __importDefault(require("node:zlib"));
const gluesdk = __importStar(require("@aws-sdk/client-glue"));
const avro = __importStar(require("avsc"));
const uuid = __importStar(require("uuid"));
var SchemaType;
(function (SchemaType) {
    SchemaType["AVRO"] = "AVRO";
})(SchemaType || (exports.SchemaType = SchemaType = {}));
var SchemaCompatibilityType;
(function (SchemaCompatibilityType) {
    SchemaCompatibilityType["NONE"] = "NONE";
    SchemaCompatibilityType["BACKWARD"] = "BACKWARD";
    SchemaCompatibilityType["BACKWARD_ALL"] = "BACKWARD_ALL";
    SchemaCompatibilityType["DISABLED"] = "DISABLED";
    SchemaCompatibilityType["FORWARD"] = "FORWARD";
    SchemaCompatibilityType["FORWARD_ALL"] = "FORWARD_ALL";
    SchemaCompatibilityType["FULL"] = "FULL";
    SchemaCompatibilityType["FULL_ALL"] = "FULL_ALL";
})(SchemaCompatibilityType || (exports.SchemaCompatibilityType = SchemaCompatibilityType = {}));
var ERROR;
(function (ERROR) {
    ERROR[ERROR["NO_ERROR"] = 0] = "NO_ERROR";
    ERROR[ERROR["INVALID_HEADER_VERSION"] = 1] = "INVALID_HEADER_VERSION";
    ERROR[ERROR["INVALID_COMPRESSION"] = 2] = "INVALID_COMPRESSION";
    ERROR[ERROR["INVALID_SCHEMA_ID"] = 3] = "INVALID_SCHEMA_ID";
    ERROR[ERROR["INVALID_SCHEMA"] = 4] = "INVALID_SCHEMA";
})(ERROR || (exports.ERROR = ERROR = {}));
class GlueSchemaRegistry {
    /*
  This class aims to be compatible with the java serde implementation from AWS.
  https://github.com/awslabs/aws-glue-schema-registry/blob/master/serializer-deserializer/src/main/java/com/amazonaws/services/schemaregistry/serializers/SerializationDataEncoder.java
  https://github.com/awslabs/aws-glue-schema-registry/blob/master/common/src/main/java/com/amazonaws/services/schemaregistry/utils/AWSSchemaRegistryConstants.java
  */
    gc;
    registryName;
    glueSchemaIdCache;
    avroSchemaCache;
    /**
     * Constructs a GlueSchemaRegistry
     *
     * @param registryName - name of the Glue registry you want to use
     * @param props - optional AWS properties that are used when constructing the Glue object from the AWS SDK
     */
    constructor(registryName, props) {
        this.gc = new gluesdk.GlueClient(props);
        this.registryName = registryName;
        this.glueSchemaIdCache = {};
        this.avroSchemaCache = {};
    }
    /**
     * Updates the Glue client. Useful if you need to update the credentials, for example.
     *
     * @param props settings for the AWS Glue client
     */
    updateGlueClient(props) {
        this.gc = new gluesdk.GlueClient(props);
    }
    async loadGlueSchema(schemaId) {
        return await this.gc.send(new gluesdk.GetSchemaVersionCommand({
            SchemaVersionId: schemaId
        }));
    }
    /**
     * Creates a new schema in the AWS Glue Schema Registry.
     * Note: do not use createSchema if you want to create a new version of an existing schema.
     * Instead use register().
     *
     * @param props - the details about the schema
     * @throws if the schema already exists
     * @throws if the Glue compatibility check fails
     */
    async createSchema(props) {
        const res = await this.gc.send(new gluesdk.CreateSchemaCommand({
            DataFormat: props.type,
            Compatibility: props.compatibility,
            SchemaName: props.schemaName,
            SchemaDefinition: props.schema,
            RegistryId: {
                RegistryName: this.registryName
            }
        }));
        if (res.SchemaVersionStatus === 'FAILURE')
            throw new Error('Schema registration failure');
        return res.SchemaVersionId;
    }
    /**
     * Registers a new version of an existing schema.
     * Returns the id of the existing schema version if a similar version already exists.
     *
     * @param props - the details about the schema
     * @returns {string} the id of the schema version
     * @throws if the schema does not exist
     * @throws if the Glue compatibility check fails
     */
    async register(props) {
        const hash = node_crypto_1.default.createHash('SHA256').update(props.schemaName + '.' + props.schema);
        const hashString = hash.digest('hex').toString();
        const cachehit = this.glueSchemaIdCache[hashString];
        if (cachehit) {
            return cachehit;
        }
        const schema = await this.gc.send(new gluesdk.RegisterSchemaVersionCommand({
            SchemaDefinition: props.schema,
            SchemaId: {
                RegistryName: this.registryName,
                SchemaName: props.schemaName
            }
        }));
        if (!schema.SchemaVersionId)
            throw new Error('Schema does not have SchemaVersionId');
        if (schema.Status === 'FAILURE')
            throw new Error('Schema registration failure');
        this.glueSchemaIdCache[hashString] = schema.SchemaVersionId;
        // store the avro schema in its cache to avoid another glue lookup when it's used
        const avroSchema = avro.Type.forSchema(JSON.parse(props.schema));
        this.avroSchemaCache[schema.SchemaVersionId] = avroSchema;
        return schema.SchemaVersionId;
    }
    static COMPRESSION_DEFAULT = 0;
    static COMPRESSION_ZLIB = 5;
    static HEADER_VERSION = 3;
    static HEADER_VERSION_BYTE = GlueSchemaRegistry.initByteBuffer(GlueSchemaRegistry.HEADER_VERSION); // default version 3
    static COMPRESSION_DEFAULT_BYTE = GlueSchemaRegistry.initByteBuffer(GlueSchemaRegistry.COMPRESSION_DEFAULT // no compression
    );
    static COMPRESSION_ZLIB_BYTE = GlueSchemaRegistry.initByteBuffer(GlueSchemaRegistry.COMPRESSION_ZLIB);
    /**
     * Encode the object with a specific glue schema version
     *
     * @param glueSchemaId - UUID of the Glue schema version that should be used to encode the message
     * @param object - the object to encode
     * @param props - optional encoding options
     * @returns - a Buffer containing the binary message
     */
    async encode(glueSchemaId, object, props) {
        const ZLIB_COMPRESS_FUNC = (buf) => {
            return new Promise((resolve, reject) => {
                node_zlib_1.default.deflate(buf, (err, data) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(data);
                    }
                });
            });
        };
        const NO_COMPRESS_FUNC = (buf) => Promise.resolve(buf);
        const schema = await this.getAvroSchemaForGlueId(glueSchemaId);
        // construct the message binary
        const buf = schema.toBuffer(object);
        let compression_func = ZLIB_COMPRESS_FUNC;
        let compressionbyte = GlueSchemaRegistry.COMPRESSION_ZLIB_BYTE;
        if (props && !props.compress) {
            compression_func = NO_COMPRESS_FUNC;
            compressionbyte = GlueSchemaRegistry.COMPRESSION_DEFAULT_BYTE;
        }
        return Buffer.concat([GlueSchemaRegistry.HEADER_VERSION_BYTE, compressionbyte, this.convertUUIDstringToByteArray(glueSchemaId), await compression_func(buf)]);
    }
    /**
     * Analyze the binary message to determine if it is valid and if so, what schema version it was encoded with.
     *
     * @param message - the binary message to analyze
     * @returns - an object containing the analysis results @see AnalyzeMessageResult
     */
    async analyzeMessage(message) {
        const headerversion = message.readInt8(0);
        if (headerversion !== GlueSchemaRegistry.HEADER_VERSION) {
            return {
                valid: false,
                error: ERROR.INVALID_HEADER_VERSION
            };
        }
        const compression = message.readInt8(1);
        if (compression !== GlueSchemaRegistry.COMPRESSION_DEFAULT && compression !== GlueSchemaRegistry.COMPRESSION_ZLIB) {
            return {
                valid: false,
                error: ERROR.INVALID_COMPRESSION
            };
        }
        try {
            const producerSchemaId = uuid.stringify(message, 2);
            try {
                const producerschema = await this.loadGlueSchema(producerSchemaId);
                if (!producerschema)
                    throw new Error('Schema not found');
                if (producerschema.Status === 'FAILURE') {
                    return {
                        valid: false,
                        error: ERROR.INVALID_SCHEMA
                    };
                }
                return {
                    valid: true,
                    headerversion,
                    compression,
                    schemaId: producerSchemaId,
                    schema: producerschema
                };
            }
            catch {
                return {
                    valid: false,
                    error: ERROR.INVALID_SCHEMA
                };
            }
        }
        catch {
            return {
                valid: false,
                error: ERROR.INVALID_SCHEMA_ID
            };
        }
    }
    /**
     * Decode a message with a specific schema.
     *
     * @param message - Buffer with the binary encoded message
     * @param consumerschema - The Avro schema that should be used to decode the message
     * @returns - the deserialized message as object
     */
    async decode(message, consumerschema) {
        const headerversion = message.readInt8(0);
        const compression = message.readInt8(1);
        if (headerversion !== GlueSchemaRegistry.HEADER_VERSION) {
            throw new Error(`Only header version ${GlueSchemaRegistry.HEADER_VERSION} is supported, received ${headerversion}`);
        }
        if (compression !== GlueSchemaRegistry.COMPRESSION_DEFAULT && compression !== GlueSchemaRegistry.COMPRESSION_ZLIB) {
            throw new Error(`Only compression type 0 and 5 are supported, received ${compression}`);
        }
        const ZLIB_UNCOMPRESS_FUNC = (buf) => {
            return new Promise((resolve, reject) => {
                node_zlib_1.default.inflate(buf, (err, data) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(data);
                    }
                });
            });
        };
        const NO_UNCOMPRESS_FUNC = (buf) => Promise.resolve(buf);
        const producerSchemaId = uuid.stringify(message, 2);
        const producerschema = await this.getAvroSchemaForGlueId(producerSchemaId);
        const resolver = this.getResolver(producerschema, consumerschema);
        const content = Buffer.from(message.subarray(18));
        let handlecompression = NO_UNCOMPRESS_FUNC;
        if (compression === GlueSchemaRegistry.COMPRESSION_ZLIB) {
            handlecompression = ZLIB_UNCOMPRESS_FUNC;
        }
        return consumerschema.fromBuffer(await handlecompression(content), resolver);
    }
    async getAvroSchemaForGlueId(id) {
        if (this.avroSchemaCache[id])
            return this.avroSchemaCache[id];
        const schemastring = (await this.loadGlueSchema(id)).SchemaDefinition;
        if (!schemastring)
            throw new Error('Glue returned undefined schema definition');
        const schema = avro.Type.forSchema(JSON.parse(schemastring));
        this.avroSchemaCache[id] = schema;
        return schema;
    }
    convertUUIDstringToByteArray(id) {
        const idasbytes = uuid.parse(id);
        return new Uint8Array(idasbytes);
    }
    getResolver(producerschema, consumerschema) {
        return consumerschema.createResolver(producerschema);
    }
    static initByteBuffer(value) {
        return Buffer.from([value]);
    }
}
exports.GlueSchemaRegistry = GlueSchemaRegistry;
//# sourceMappingURL=glue-schema-registry.js.map