import prettier from 'eslint-plugin-prettier/recommended'
import sonarjs from 'eslint-plugin-sonarjs'
import eslint from '@eslint/js'
import tseslint from 'typescript-eslint'
import unusedImports from 'eslint-plugin-unused-imports'
import simpleImportSort from 'eslint-plugin-simple-import-sort'
import sort from 'eslint-plugin-sort'
import pluginSecurity from 'eslint-plugin-security'
import globals from 'globals'

export default [
  {
    ignores: [
      '**/node_modules/',
      '**/apidoc/',
      '**/dist/',
      '**/build/',
      '**/test/',
      '**/package-lock.json',
      '**/.env',
      '**/.gitignore',
      'src/test.ts',
      '**/thunder-tests/',
      '**/tsconfig.json',
      '**/seed.router-data.json',
      '**/seed.router-data2.json',
      '**/test.js',
      '**/testSlack.js',
      'src/models/generated/',
      'src/models/generated-joi-schemas/',
      '**/.eslintrc.json',
      '**/prisma/',
      '**/*.json',
      '**/.rnd/',
      'eslint.config.mjs',
      '**/*.test.ts',
      'test1.ts'
    ]
  },
  sonarjs.configs.recommended,
  prettier,
  eslint.configs.recommended,
  ...tseslint.configs.recommended,
  pluginSecurity.configs.recommended,
  sort.configs['flat/recommended'],
  {
    plugins: {
      '@typescript-eslint': tseslint.plugin,
      'unused-imports': unusedImports,
      'simple-import-sort': simpleImportSort
    },

    languageOptions: {
      globals: {
        ...globals.node
      },

      parser: tseslint.parser,
      ecmaVersion: 'latest',
      sourceType: 'module'
    }
  },
  {
    files: ['**/*.ts'],

    rules: {
      indent: [
        'error',
        4,
        {
          SwitchCase: 1
        }
      ],

      '@typescript-eslint/no-explicit-any': 'off',
      'prettier/prettier': 'warn',
      'no-console': 'warn',
      'linebreak-style': ['error', 'unix'],

      quotes: [
        'error',
        'single',
        {
          allowTemplateLiterals: true
        }
      ],

      semi: ['error', 'never'],
      'sort/import-members': 'off',
      eqeqeq: 'warn',
      'sort/object-properties': 'off',
      'no-fallthrough': 'off',
      '@typescript-eslint/no-unused-vars': 'off',
      'sonarjs/content-length': 'warn',
      'sonarjs/cognitive-complexity': ['warn', 20],
      'sonarjs/no-small-switch': 'warn',
      'sonarjs/no-duplicate-string': 'off',
      'sonarjs/no-identical-functions': ['warn', 5],
      'sonarjs/no-nested-functions': 'warn',
      'sonarjs/use-type-alias': 'off',
      '@typescript-eslint/no-inferrable-types': 'off',
      'object-shorthand': ['warn', 'always'],
      'no-unused-vars': 'off',
      'unused-imports/no-unused-imports': 'error',
      'unused-imports/no-unused-vars': [
        'warn',
        {
          vars: 'all',
          varsIgnorePattern: '^_',
          args: 'after-used',
          argsIgnorePattern: '^_'
        }
      ],
      'sort/imports': 'off',
      'simple-import-sort/imports': 'error',
      'simple-import-sort/exports': 'error'
    }
  },
  {
    files: ['**/*.test.ts', '**/*.spec.ts', 'src/common/test.ts'],

    rules: {
      '@typescript-eslint/no-unused-vars': 'off',
      'sonarjs/no-identical-functions': 'off'
    }
  },
  {
    files: ['**/*.json'],

    rules: {
      'sonarjs/no-duplicate-string': 'off',
      'sort/object-properties': 'off'
    }
  }
]
